/*global define, window, ep, reset*/
/*jslint nomen:true*/
define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!./twitter-test.html',
    'jquery',

    './twitter',
    '$ready!'
], function (QUnit, fixture, $) {
    'use strict';

    QUnit.module('de_epages/externalcontent/ui/twitter', {
        setup: function () {
            fixture.init();

            ep.config = {
                javascriptRoot: '.',
                debugLevel: 2,
                canonicalUrl: 'www.canonical.de',
                webUrl: 'www.webUrl.de'
            };

            // Twitter *TWTR* API for testing.
            window.TWTR = {
                Widget: function (params) {
                    var self = this;
                    this.render = function () {
                        window.isrendered = true;
                        return self;
                    };
                    this.start = function () {
                        window.isstarted = true;
                        return self;
                    };
                    this.setUser = function (username) {
                        window.username = username;
                        return self;
                    };
                    this.destroy = function () {
                        window.isdestroyed = true;
                    };
                    window.params = params;
                }
            };

            // Twitter *twttr* API for testing.
            window.twttr = {
                widgets: {
                    load: function () {
                        window.isloaded = true;
                    }
                }
            };

            // Reset test values.
            window.reset = function () {
                window.wasrendered = false;
                window.wasstarted = false;
                window.username = "";
                window.isdestroyed = false;
                window.isloaded = false;
                window.params = {};
            };
        },

        teardown: fixture.destroy
    });

    QUnit.test('Test twitter methods', 3, function () {
        $.each(['_create', '_twitterButton', 'destroy'], function (i, fn) {
            QUnit.equal(
                $.isFunction($.ui.externalcontentUiTwitter.prototype[fn]),
                true,
                fn + ' is a method of twitter'
            );
        });
    });

    QUnit.test('Test twitter', 7, function () {
        // Overwrite *ajax* for testing.
        $.ajax = function (o) {
            return {
                done: function (_func) {
                    _func();
                }
            };
        };

        // Create *twitter* (share) gadget. (twttr API, see above)
        $("#twitterbutton1").externalcontentUiTwitter({
            url: "www.testurl1.de",
            type: "share"
        });

        QUnit.equal(
            window.isloaded,
            true,
            "the Twitter function loading the share button in the target div has been called"
        );

        QUnit.equal(
            $("#twitterbutton1").find("a").attr("class"),
            "twitter-share-button",
            "the appended anchor has the class twitter-share-button"
        );

        QUnit.equal(
            $("#twitterbutton1").find("a").attr("data-url"),
            "www.testurl1.de",
            "the passed url-parameter has been added into the data-url attribute"
        );

        reset();

        // Create *twitter* (follow) gadget. (twttr API, see above)
        $("#twitterbutton2").externalcontentUiTwitter({
            username: "Boobs",
            type: "follow"
        });

        QUnit.equal(
            window.isloaded,
            true,
            "the Twitter function loading the follow button in the target div has been called"
        );

        QUnit.equal(
            $("#twitterbutton2").find("a").attr("class"),
            "twitter-follow-button",
            "the appended anchor has the class twitter-follow-button"
        );

        QUnit.equal(
            $("#twitterbutton2").find("a").attr("href"),
            "https://twitter.com/Boobs",
            "the passed username-parameter has been added into the visible anchor font"
        );

        reset();

        // Destroy *twitter* (follow) gadget.
        $("#twitterbutton2").externalcontentUiTwitter("destroy");

        QUnit.equal(
            $("#twitterbutton2").children().length,
            0,
            "The container div was removed from *self.element*"
        );

        reset();
    });
});