/**
 * Create an drag and drop uploader.
 *
 * The `.presentationUiDnduploader()` method creates a drag and drop uploader.
 *
 * Only applicable to <input type="file"> nodes.
 *
 * ### Examples
 * Explanation for example,
 *
 * JavaScript:
 *
 *
 *     <input type="file" name="NewFile" class="ep-js" data-js="de_epages.presentationUiDnduploader({
 *       multiple:false,
 *       accept:'image/*',
 *       data:{
 *         ChangeAction:'SaveImage',
 *         ObjectID:#ObjectID,
 *         SCALE_WIDTH_l:#Shop.ProductImageLargeWidth,
 *         SCALE_HEIGHT_l:#Shop.ProductImageLargeHeight,
 *         SCALE_WIDTH_m:200,
 *         SCALE_WIDTH_s:100,
 *         SCALE_HEIGHT_s:100,
 *         SCALE_WIDTH_h:150,
 *         SCALE_HEIGHT_h:150,
 *         SCALE_WIDTH_xs:50,
 *         SCALE_HEIGHT_xs:33,
 *         SCALE_NAME_m:'ImageMedium',
 *         SCALE_NAME_s:'ImageSmall',
 *         SCALE_NAME_h:'ImageHotDeal'
 *       },
 *       callback:'?ViewAction='+ep.config.viewAction+'&ObjectID='+ep.config.objectId+'&DialogArea=ViewArea'
 *     })"/>
 *
 *
 * @class jQuery.ui.presentationUiDnduploader
 * @extends jQuery.widget
 *
 * @uses jQuery.dict
 * @uses jQuery.tmpl
 * @uses jQuery.i18n
 * @uses jQuery.expr
 * @uses jQuery.fn.form
 * @uses jQuery.ui.widget
 * @uses jQuery.ui.progressbar
 * @uses de_epages
 * @uses ep.ajaxTransport
 * @uses ep.ui.dialog
 * @since 6.17.38
 */

/**
 * @cfg {String} [accept] A string of MIME types and/or MIME type groups.
 */

/**
 * @cfg {String} [url] A string containing the URL to which the request is sent.
 */

/**
 * @cfg {String} [buttonText] A string for the upload button
 */

/**
 * @cfg {Object} [data] A map of additional data to send with the request.
 */

/**
 * @cfg {Function,String} [callback] A url to open or method to call when upload process is finished. The callback method receives data and statusText as arguments.
 */

/**
 * @cfg {Boolean} [multiple] A boolean indication whether to allow muplitple files to upload.
 */

/**
 * @cfg {Boolean} [big] A boolean boolean short handle set style big.
 */

/**
 * @cfg {String} [style] One of the available styles: normal,big,icon.
 */

/**
 * @cfg {Array} [exists] An array including already existing files.
 */

/**
 * See `jQuery.ui.presentationUiDnduploader` for details.
 *
 * @param {Object} [options] A map of additional options pass to the method.
 * @param {String} [accept] A string of MIME types and/or MIME type groups.
 * @param {String} [url] A string containing the URL to which the request is sent.
 * @param {String} [buttonText] A string for the upload button
 * @param {Object} [data] A map of additional data to send with the request.
 * @param {Function,String} [callback] A url to open or method to call when upload process is finished. The callback method receives data and statusText as arguments.
 * @param {Boolean} [multiple] A boolean indication whether to allow muplitple files to upload.
 * @param {Boolean} [big] A boolean boolean short handle set style big.
 * @param {String} [style] One of the available styles: normal,big,icon.
 * @param {Array} [exists] An array including already existing files.
 *
 * @method presentationUiDnduploader
 * @member jQuery
 *
 * @since 6.12.0
 */

/*
 * @copyright       © Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages.presentation.ui.dnduploader
 */

define( "de_epages/presentation/ui/dnduploader", [
    "jquery",
    "ep",
    "de_epages",
    "util/mime",
    "$dict!../dictionary",
    "$tmpl!./dnduploader",
    "$tmpl!./dnduploader-inputs",
    "$tmpl!./dnduploader-sortable_item",

    "jquery/tmpl",
    "jquery/ui/sortable"
    // "jquery/ui/widget",
    // "jquery/i18n",
    // "jquery/expr",
    // "jquery/cookie",
    // "ep/ajax",
    // "ep/ui/input",
    // "ep/ui/form",
    // "ep/ui/dialog"
], function ($, ep, de_epages, mime, presentationDict, tmplDndUploader, tmplDndUploaderInputs, tmplDndUploaderSortableItem) {

    /*
     * @dictionary      ep.dict
     *
     * @translation     {Cancel}
     *                  {Ignore}
     *                  {Replace}
     *
     * @dictionary      de_epages.presentation.dictionary
     *
     * @translation     {FileUploadSingle}
     *                  {FileUploadMultiple}
     *                  {UploaderError}
     *                  {UploaderReplace}
     *                  {FixErrors}
     *                  {FilesUploaded}
     *                  {FileSizeLimitError}
     *                  {InternalServerError}
     *                  {FontLicenseRestricted}
     *                  {MimetypeNotAcceptError}
     *                  {UnknownError}
     *                  {FeatureLimitExceeded}
     *                  {InvalidMP3File}
     *                  {InvalidImageFile}
     *                  {ImageExceedsMegapixels}
     *                  {VirusFoundError}
     *                  {FileAlreadyExistsOnServer}
     *                  {InvalidImageSize}
     */

    // var undef,

    //     tCancel = presentationDict.translate('Cancel'),
    //     tIgnore = presentationDict.translate('Ignore'),
    //     tReplace = presentationDict.translate('ReplaceFiles'),
    //     tUploadFile = presentationDict.translate('FileUploadSingle'),
    //     tUploadFiles = presentationDict.translate('FileUploadMultiple'),
    //     tUploaderError = presentationDict.translate('UploaderError'),
    //     tUploaderReplace = presentationDict.translate('FileUploadMultiple'),
    //     tFixErrors = presentationDict.translate('FixErrors'),
    //     tFilesUploaded = presentationDict.translate('FilesUploaded'),
    //     tAlreadyExists = presentationDict.translate('FileAlreadyExistsOnServer');

    $.widget('ui.presentationUiDnduploader', {
        files: {
            order: [],
            exist: [],
            upload: [],
            error: []
        },

        // fileCount,

        options: {
        //    files :{exists: [], order: []},
            mime: 'image.*',
            join: ',',
            multiple: true,
            callback: $.noop,
            url: '?',
            // tokenName: 'SecToken',
            data: {
                'ViewAction': 'JSONViewResponse'
            },

            dimensions: {
                width: 150,
                height: 100
            }
        },

        _create: function () {
            var self = this,
                elem = self.element,
                o = self.options,
                renderedTmpl;


            renderedTmpl = tmplDndUploader([{
                multiple: o.multiple,
                order: (o.files && o.files.order) ? o.files.order.join(o.join) : ""
            }]);

            // initialize object for dropZone elements
            self.dropZone = {};

            // load template and push named tmpl elements to current instance
            $.extend(self.dropZone, renderedTmpl.tmplItem('elements'));

            elem.append(renderedTmpl);
            // elem.html(renderedTmpl);

            // register handler for drag and drop and file input
            // DragOver-Handler
            self.dropZone.area.on('dragover', $.proxy(function (e) {
                self._handleDragOver(e);
            }, self));
            // Drop-Handler
            self.dropZone.area.on('drop', $.proxy(function (e) {
                self._handleDrop(e);
            }, self));
            // fileInput-Handler
            self.dropZone.fileInput.on('change', $.proxy(function (e) {
                self._handleFile(e);
            }, self));

            // initialize sortable list
            self._initSortable();

            // init options
            // self._setOptions({
            //     'accept': mime.mime(o.accept || self.elem.attr('accept') || '*/*'),
            //     'multiple': o.multiple || self.elem.attr('multiple') || false
            // });
        },

        _initSortable: function () {
            var self = this,
                o = self.options;

            // initialize sortable list
            self.dropZone.sortable.sortable({
                placeholder: "ep-dropzone-state-highlight",
                items: "li:not(.ep-dropzone-placeholder)",
                stop: function(event, ui) {
                    // console.log('sortable stop');
                    // var ul = ui.item.closest('ul'),
                    //     list = ul.children('li');

                    // // Sortier-Array zurücksetzen
                    // fileObject.sort.length = 0;
                    // // Sortier-Array neu aufbauen (aus Reihenfolge der LIs)
                    // list.each(function(key, item){
                    //     fileObject.sort.push($(item).data('fileid'));
                    // });

                    // rebuild order array
                    self._sortOrder(self.dropZone.sortable.find('li.ep-dropzone-sortable-item'));
                }
            });
            self.dropZone.sortable.disableSelection();
        },

        _sortOrder: function (list) {
            var self = this
                o = self.options;

            // reset order array
            self.files.order.length = 0;
            // rebuild order array
            list.each(function(key, item){
                self.files.order.push($(item).data('id'));
            });

            self.dropZone.orderInput.val(self.files.order.join(o.join));

            // console.log("order: ", self.files.order);
        },

        // DnD
        _handleDragOver: function(evt) {
            evt.stopPropagation();
            evt.preventDefault();

            // set effect to copy
            evt.originalEvent.dataTransfer.dropEffect = 'copy';
        },

        // Drop-Handler
        _handleDrop: function(evt) {
            evt.stopPropagation();
            evt.preventDefault();

            var self = this;

            self._manageFiles(evt.originalEvent.dataTransfer.files);
        },

        _handleFile: function(evt) {
            var self = this;

            self._manageFiles(evt.originalTarget.files);
        },

        _manageFiles: function (files) {
            var self = this,
                o = self.options,
                len = o.multiple ? files.length : 1,
                i;

            // clear error array
            self.files.error.length = 0;
            self.fileCount = len;

            for (i=0; i<len; i++) {
                self._fileReader(files[i]);
            }
        },

        // FileReader
        _fileReader: function(file) {
            var self = this,
                reader = new FileReader();

            reader.onload = function (evt) {
                var fileReader = this;

                // Handler aufrufen, um die Datei weiter zu verarbeiten
                self._readerSuccessHandler(fileReader, file, evt);
            };

            // reader.onprogress = readerProgressHandler;
            // reader.onerror = readerErrorHandler;

            // read file as dataUrl
            reader.readAsDataURL(file);

        },

        _readerSuccessHandler: function(fileReader, file, evt) {
            var self = this,
                o = self.options,
                id = "id_" + file.size + new Date().getTime(),
                fileData;

            if (!file.type.match(o.mime)) {
                self.files.error.push({
                    id: id,
                    name: file.name
                });

                self.fileCount--;

                return;
            }

            fileData = {
                id: id,
                name: file.name,
                dataUrl: fileReader.result
                // ,file: file
            };

            // write informations for file on object
            self.files.upload.push(fileData);

            var $img = $('<img />');

            // write dataUrl to image source
            $img.attr('src', fileReader.result);

            // load handler for image
            $img.on('load', function(evt) {
                var $li = $('<li class="ep-dropzone-sortable-item ep-center-align" />'),
                    $img = $(this),
                    $body = $('body'),
                    dimensions = {},
                    itemTmpl,
                    inputTmpl,
                    elements = {};

                itemTmpl = tmplDndUploaderSortableItem([{
                    id: id
                }]);
                // load template and push named tmpl elements to object
                $.extend(elements, itemTmpl.tmplItem('elements'));

                // register click handler for delete button
                elements.deleteButton.on('click', $.proxy(function (evt) {
                    self._deleteItemById($(evt.target).data('id'));
                }, self));

                $img.addClass('ep-dropzone-image-offset');
                $body.append($img);

                // Dimensionsobjekt erstellen
                dimensions.width = $img.width();
                $img.data('width', dimensions.width);
                dimensions.height = $img.height();
                $img.data('height', dimensions.height);

                // Dimensionen berechnen (um das Bild passend für die DropZone zu machen)
                dimensions = self._recalcDimensions(dimensions);

                $img.width(dimensions.width);
                $img.height(dimensions.height);

                $img.removeClass('ep-dropzone-image-offset');

                itemTmpl.append($img);
                self.dropZone.placeholder.before(itemTmpl);
                // self.dropZone.sortable.append($li);
                // self.dropZone.sortable.sortable('refresh');

                //
                inputTmpl = tmplDndUploaderInputs([fileData]);
                self.dropZone.holder.append(inputTmpl);

                self.fileCount--;
                // an error occurs after the last uploaded element
                if (self.fileCount === 0) {
                    self.dropZone.sortable.sortable('refresh');
                    // rebuild order array
                    self._sortOrder(self.dropZone.sortable.find('li.ep-dropzone-sortable-item'));

                    // ToDo: Array sortieren

                    if (self.files.error.length) {
                        alert('Es können nicht alle Dateien hochgeladen werden.');
                    }
                }

                // console.log("li's: ", self.dropZone.sortable.find('li').length - 1, self.files.upload.length)
                // if ((self.dropZone.sortable.find('li').length - 1 === self.files.upload.length) && self.files.error.length) {
                //     alert('Es können nicht alle Dateien hochgeladen werden.');
                // }
            });
        },

        _readerProgressHandler: function() {
            // ToDo: Handler für die Fortschrittsanzeige
            //console.log("readerProgressHandler: ", arguments);
        },

        _readerErrorHandler: function() {
            // ToDo: Handler für den Fehlerfall
            //console.log("readerErrorHandler: ", arguments);
        },

        // Helper
        _recalcDimensions: function(dimensions) {
            var self = this,
                o = self.options,
                factor;

            dimensions.origWidth = dimensions.width;
            dimensions.origHeight = dimensions.height;

            if (dimensions.width > o.dimensions.width) {
                factor = o.dimensions.width / dimensions.width;

                dimensions.width = o.dimensions.width;
                dimensions.height = dimensions.height * factor;
            }

            if (dimensions.height > o.dimensions.height) {
                factor = o.dimensions.height / dimensions.height;

                dimensions.height = o.dimensions.height;
                dimensions.width = dimensions.width * factor;
            }

            dimensions.width = Math.floor(dimensions.width);
            dimensions.height = Math.floor(dimensions.height);

            return dimensions;
        },

        // _setOption: function (name, value) {
        //     var o = this.options;

        //     if (value !== undef) {
        //         switch (name) {
        //         case 'accept':
        //             this.elem.attr(name, mime.mime(value));
        //             break;
        //         case 'multiple':
        //             this.elem.attr(name, value);
        //             this._setOption('style', o.style);
        //             break;
        //         }
        //     }

        //     return this._superApply(arguments);
        // },

        _upload: function (event, doReplace) {
            var self = this,
                o = self.options;
        },

        _getItemById: function (id) {
            var self = this,
                i;

            // loop over already existing images
            for (i=0; i<self.files.exist.length; i++) {
                if (self.files.exist[i].id === id){
                    return self.files.exist[i];
                }
            }

            // loop over new images
            for (i=0; i<self.files.upload.length; i++) {
                if (self.files.upload[i].id === id){
                    return self.files.upload[i];
                }
            }

            return false;
        },

        _deleteItemById: function (id) {
            var self = this,
                i;

            // console.log('delete: ', id);
            // // loop over already existing images
            // for (i=0; i<self.files.exist.length; i++) {
            //     if (self.files.exist[i].id === id){
            //         return self.files.exist[i];
            //     }
            // }

            // // loop over new images
            // for (i=0; i<self.files.upload.length; i++) {
            //     if (self.files.upload[i].id === id){
            //         return self.files.upload[i];
            //     }
            // }

            return false;
        },

        /**
         * This method removes the container on which the Uploader has been appended to.
         *
         * @method destroy
         * @member jQuery.ui.presentationUiUploader
         *
         * @since 6.12.0
         */
        destroy: function () {
            var self = this;

            self.elem.off('.presentationUiDnduploader');

            self._superApply(arguments);
        }

    });

    return de_epages;

});
